@@ -20,8 +20,8 @@ from types import SimpleNamespace
 from typing import Any, Dict, List, Optional, Tuple, Union
 
 import torch
+import tvm_ffi
 
-from ..artifacts import ArtifactPath, MetaInfoHash
 from ..autotuner import (
     AutoTuner,
     DynamicTensorSpec,
@@ -29,17 +29,16 @@ from ..autotuner import (
     TunableRunner,
     TuningConfig,
 )
-from ..jit import JitSpec
-from ..jit import env as jit_env
+from ..jit.core import logger
 from ..jit import (
-    gen_jit_spec,
     setup_cubin_loader,
-    sm90a_nvcc_flags,
-    current_compilation_context,
 )
-from ..jit.cpp_ext import is_cuda_version_at_least
-from ..jit.cubin_loader import get_cubin
-from ..jit.cutlass_gemm.generate_kernels import generate_gemm_operations
+from ..jit.fused_moe import (
+    gen_cutlass_fused_moe_sm120_module,
+    gen_cutlass_fused_moe_sm100_module,
+    gen_cutlass_fused_moe_sm90_module,
+    gen_trtllm_gen_fused_moe_sm100_module,
+)
 from ..utils import (
     check_shape_dtype_device,
     device_support_pdl,
@@ -261,147 +260,14 @@ def convert_to_block_layout(input_tensor: torch.Tensor, blockK: int) -> torch.Te
     return input_tensor.view(M, K // blockK, blockK).permute(1, 0, 2).contiguous()
 
 
-def gen_cutlass_fused_moe_sm100_module(use_fast_build: bool = False) -> JitSpec:
-    nvcc_flags = [
-        "-DCOMPILE_BLACKWELL_TMA_GEMMS",
-        "-DCOMPILE_BLACKWELL_TMA_GROUPED_GEMMS",
-        "-DENABLE_BF16",
-        "-DENABLE_FP8",
-        "-DENABLE_FP4",
-        "-DUSING_OSS_CUTLASS_MOE_GEMM",
-    ]
-
-    nvcc_flags += current_compilation_context.get_nvcc_flags_list(
-        supported_major_versions=[10, 11, 12]
-    )
-
-    return gen_cutlass_fused_moe_module(nvcc_flags, "100", use_fast_build)
-
-
-def gen_cutlass_fused_moe_sm90_module(use_fast_build: bool = False) -> JitSpec:
-    nvcc_flags = sm90a_nvcc_flags + [
-        "-DCOMPILE_HOPPER_TMA_GEMMS",
-        "-DCOMPILE_HOPPER_TMA_GROUPED_GEMMS",
-        "-DENABLE_BF16",
-        "-DENABLE_FP8",
-        "-DENABLE_FP4" if is_cuda_version_at_least("12.8") else "",
-        "-DUSING_OSS_CUTLASS_MOE_GEMM",
-    ]
-    return gen_cutlass_fused_moe_module(nvcc_flags, "90", use_fast_build)
-
-
-def gen_cutlass_fused_moe_module(
-    nvcc_flags: List[str], device_arch: str, use_fast_build: bool = False
-) -> JitSpec:
-    """
-    Generate a JitSpec for the cutlass fused moe module.
-    """
-    output_dir = (
-        jit_env.FLASHINFER_CSRC_DIR
-        / f"nv_internal/tensorrt_llm/cutlass_instantiations/{device_arch}"
-    )
-
-    try:
-        # Create output directory if it doesn't exist
-        output_dir.mkdir(parents=True, exist_ok=True)
-
-        generate_gemm_operations(
-            output_dir,
-            f"{device_arch};{device_arch}-real",
-        )
-
-    except Exception as e:
-        raise RuntimeError(f"Failed to generate Cutlass kernels: {e}") from e
-
-    return gen_jit_spec(
-        f"fused_moe_{device_arch}",
-        [
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_tma_warp_specialized_input.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_fp8_uint4.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_fp8_fp8.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_fp8_fp4.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_fp4_fp4.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_fp32_fp32.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_fp16_uint8.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_fp16_uint4.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_fp16_fp16.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_bf16_uint8.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_bf16_uint4.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_bf16_fp8.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_bf16_bf16.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_bf16_fp4.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/moe_gemm/moe_gemm_kernels_fp16_fp4.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/fp8_blockscale_gemm/fp8_blockscale_gemm_stub.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "fused_moe/cutlass_backend/flashinfer_cutlass_fused_moe_sm100_ops.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "fused_moe/cutlass_backend/cutlass_fused_moe_instantiation.cu",
-            # Add all generated kernels
-            *(output_dir / kernel for kernel in output_dir.rglob("*.generated.cu")),
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal/cpp/common/envUtils.cpp",
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal/cpp/common/logger.cpp",
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal/cpp/common/stringUtils.cpp",
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal/cpp/common/tllmException.cpp",
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal/cpp/common/memoryUtils.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/preQuantScaleKernel.cu",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/cutlass_kernels/cutlass_heuristic.cpp",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal/tensorrt_llm/kernels/lora/lora.cpp",
-        ],
-        extra_cuda_cflags=nvcc_flags,
-        extra_cflags=["-DFAST_BUILD"] if use_fast_build else [],
-        extra_ldflags=["-lcuda"],
-        extra_include_paths=[
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal",
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal" / "include",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal"
-            / "tensorrt_llm"
-            / "cutlass_extensions"
-            / "include",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal"
-            / "tensorrt_llm"
-            / "kernels"
-            / "cutlass_kernels"
-            / "include",
-            jit_env.FLASHINFER_CSRC_DIR
-            / "nv_internal"
-            / "tensorrt_llm"
-            / "kernels"
-            / "cutlass_kernels",
-        ],
-    )
-
-
 @functools.cache
 def get_cutlass_fused_moe_module(backend: str = "100", use_fast_build: bool = False):
-    if backend in ("100", "103", "110", "121", "122"):
-        FusedMoeRunner = gen_cutlass_fused_moe_sm100_module(
-            use_fast_build
-        ).build_and_load(class_name="FusedMoeRunner")
+    if backend in ("120", "121"):
+        module = gen_cutlass_fused_moe_sm120_module(use_fast_build).build_and_load()
+    elif backend in ("100", "103", "110"):
+        module = gen_cutlass_fused_moe_sm100_module(use_fast_build).build_and_load()
     elif backend == "90":
-        FusedMoeRunner = gen_cutlass_fused_moe_sm90_module(
-            use_fast_build
-        ).build_and_load(class_name="FusedMoeRunner")
+        module = gen_cutlass_fused_moe_sm90_module(use_fast_build).build_and_load()
     else:
         raise ValueError(f"Invalid backend: {backend}")
 
@@ -466,7 +332,7 @@ def get_cutlass_fused_moe_module(backend: str = "100", use_fast_build: bool = Fa
             )
 
             if instance_key not in MoERunner.runner_dict:
-                MoERunner.runner_dict[instance_key] = FusedMoeRunner(
+                MoERunner.runner_dict[instance_key] = module.init(
                     x_dtype,
                     weight_dtype,
                     output_dtype,
@@ -624,7 +490,29 @@ def get_cutlass_fused_moe_module(backend: str = "100", use_fast_build: bool = Fa
             if min_latency_mode
             else moe_runner.fused_moe_runner.run_moe
         )
-        result = run_moe(
+        num_active_experts_per_node = torch.empty(
+            (1,), dtype=torch.int32, device=input.device
+        )
+        experts_to_token_score = torch.empty(
+            (fc2_expert_weights.shape[0], input.shape[0]),
+            dtype=torch.float32,
+            device=input.device,
+        )
+        active_expert_global_ids = torch.empty(
+            (fc2_expert_weights.shape[0],),
+            dtype=torch.int32,
+            device=input.device,
+        )
+        min_latency_output = (
+            [
+                num_active_experts_per_node,
+                experts_to_token_score,
+                active_expert_global_ids,
+            ]
+            if min_latency_mode
+            else []
+        )
+        run_moe(
             output,
             input,
             token_selected_experts,
@@ -638,6 +526,7 @@ def get_cutlass_fused_moe_module(backend: str = "100", use_fast_build: bool = Fa
             swiglu_alpha,
             swiglu_beta,
             swiglu_limit,
+            *min_latency_output,
             tp_size,
             tp_rank,
             ep_size,
@@ -650,7 +539,16 @@ def get_cutlass_fused_moe_module(backend: str = "100", use_fast_build: bool = Fa
             enable_pdl,
         )
 
-        return result if min_latency_mode else [result]
+        return (
+            output
+            if min_latency_mode
+            else [
+                output,
+                num_active_experts_per_node,
+                experts_to_token_score,
+                active_expert_global_ids,
+            ]
+        )
 
     @register_fake_op("flashinfer::cutlass_fused_moe")
     def _fake_cutlass_fused_moe(
@@ -919,60 +817,6 @@ def cutlass_fused_moe(
 # trtllmgen-moe-fp8
 
 
-def gen_trtllm_gen_fused_moe_sm100_module() -> JitSpec:
-    # Fetch "flashinferMetaInfo.h" from the online kernel cache. This file
-    # contains the `tllmGenBatchedGemmList` as the list of available kernels
-    # online. It is included when compiling `trtllm_fused_moe_runner.cu`, etc.
-    include_path = f"{ArtifactPath.TRTLLM_GEN_BMM}/include"
-    header_name = "flashinferMetaInfo"
-
-    # use `get_cubin` to get "flashinferMetaInfo.h"
-    metainfo = get_cubin(
-        f"{include_path}/{header_name}", MetaInfoHash.TRTLLM_GEN_BMM, ".h"
-    )
-    # make sure "flashinferMetaInfo.h" is downloaded or cached
-    assert metainfo, f"{header_name}.h not found"
-
-    # currently only support Blackwell
-    nvcc_flags = current_compilation_context.get_nvcc_flags_list(
-        supported_major_versions=[10]
-    )
-
-    return gen_jit_spec(
-        "fused_moe_trtllm_sm100",
-        [
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal/cpp/common/envUtils.cpp",
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal/cpp/common/logger.cpp",
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal/cpp/common/stringUtils.cpp",
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal/cpp/common/tllmException.cpp",
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal/cpp/common/memoryUtils.cu",
-            jit_env.FLASHINFER_CSRC_DIR / "trtllm_fused_moe_kernel_launcher.cu",
-            jit_env.FLASHINFER_CSRC_DIR / "trtllm_fused_moe_runner.cu",
-            jit_env.FLASHINFER_CSRC_DIR / "trtllm_fused_moe_routing_deepseek.cu",
-            jit_env.FLASHINFER_CSRC_DIR / "trtllm_fused_moe_routing_llama4.cu",
-            jit_env.FLASHINFER_CSRC_DIR / "trtllm_fused_moe_routing_renormalize.cu",
-            jit_env.FLASHINFER_CSRC_DIR / "trtllm_fused_moe_dev_kernel.cu",
-            jit_env.FLASHINFER_CSRC_DIR / "trtllm_batched_gemm_runner.cu",
-        ],
-        extra_cuda_cflags=[
-            "-DTLLM_GEN_EXPORT_INTERFACE",
-            "-DTLLM_ENABLE_CUDA",
-            "-DENABLE_BF16",
-            "-DENABLE_FP8",
-            "-DENABLE_FP4",
-            f'-DTLLM_GEN_BMM_CUBIN_PATH=\\"{ArtifactPath.TRTLLM_GEN_BMM}\\"',
-        ]
-        + nvcc_flags,
-        extra_ldflags=["-lcuda"],
-        extra_include_paths=[
-            # link "include" sub-directory in cache
-            jit_env.FLASHINFER_CUBIN_DIR / include_path,
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal",
-            jit_env.FLASHINFER_CSRC_DIR / "nv_internal/include",
-        ],
-    )
-
-
 @functools.cache
 def get_trtllm_moe_sm100_module():
     module = gen_trtllm_gen_fused_moe_sm100_module()
@@ -1104,9 +948,14 @@ def get_trtllm_moe_sm100_module():
                 num_tokens,
             )
             if instance_key not in MoERunner.valid_tactics_dict:
-                MoERunner.valid_tactics_dict[instance_key] = (
-                    moe_op.trtllm_get_valid_moe_configs(*instance_key)
-                )
+                try:
+                    valid_tactics = moe_op.trtllm_get_valid_moe_configs(*instance_key)
+                except Exception as e:
+                    logger.debug(
+                        f"[Autotuner]: Failed to get valid tactics for {instance_key}. Error occurred: {e}"
+                    )
+                    return []
+                MoERunner.valid_tactics_dict[instance_key] = valid_tactics
             return MoERunner.valid_tactics_dict[instance_key]
 
         def forward(
@@ -1312,6 +1161,7 @@ def get_trtllm_moe_sm100_module():
         gemm1_weights_scale: torch.Tensor,
         gemm2_weights: torch.Tensor,
         gemm2_weights_scale: torch.Tensor,
+        output: torch.Tensor,
         num_experts: int,
         top_k: int,
         n_group: int,
@@ -1329,7 +1179,7 @@ def get_trtllm_moe_sm100_module():
         if enable_pdl is None:
             enable_pdl = device_support_pdl(hidden_states.device)
         # Call the C++ function for block scale MoE
-        output = moe_op.trtllm_fp8_block_scale_moe(
+        moe_op.trtllm_fp8_block_scale_moe(
             routing_logits,
             routing_bias,
             hidden_states,
@@ -1338,6 +1188,7 @@ def get_trtllm_moe_sm100_module():
             gemm1_weights_scale,
             gemm2_weights,
             gemm2_weights_scale,
+            output,
             num_experts,
             top_k,
             n_group,
@@ -1365,6 +1216,7 @@ def get_trtllm_moe_sm100_module():
         gemm1_weights_scale: torch.Tensor,
         gemm2_weights: torch.Tensor,
         gemm2_weights_scale: torch.Tensor,
+        output: torch.Tensor,
         num_experts: int,
         top_k: int,
         n_group: int,
@@ -1556,7 +1408,11 @@ def get_trtllm_moe_sm100_module():
             output,
             tactic,
         )
-
+        if isinstance(output, tvm_ffi.Array):
+            output = list(output)
+            for i in range(len(output)):
+                if isinstance(output[i], tvm_ffi.Tensor):
+                    output[i] = torch.from_dlpack(output[i])
         return output
 
     @register_fake_op("flashinfer::trtllm_fp4_block_scale_moe")
@@ -1728,6 +1584,9 @@ def trtllm_fp8_block_scale_moe(
     Returns:
         torch.Tensor: Output tensor of shape [seq_len, hidden_size]
     """
+    output = torch.empty(
+        hidden_states.shape, dtype=torch.bfloat16, device=hidden_states.device
+    )
     return get_trtllm_moe_sm100_module().trtllm_fp8_block_scale_moe(
         routing_logits,
         routing_bias,
@@ -1737,6 +1596,7 @@ def trtllm_fp8_block_scale_moe(
         gemm1_weights_scale,
         gemm2_weights,
         gemm2_weights_scale,
+        output,
         num_experts,
         top_k,
         n_group,
