@@ -23,25 +23,18 @@ from . import cubin_loader
 from . import env as env
 from .activation import gen_act_and_mul_module as gen_act_and_mul_module
 from .activation import get_act_and_mul_cu_str as get_act_and_mul_cu_str
-from .attention import cudnn_fmha_gen_module as cudnn_fmha_gen_module
+from .attention import gen_cudnn_fmha_module as gen_cudnn_fmha_module
 from .attention import gen_batch_attention_module as gen_batch_attention_module
 from .attention import gen_batch_decode_mla_module as gen_batch_decode_mla_module
 from .attention import gen_batch_decode_module as gen_batch_decode_module
 from .attention import gen_batch_mla_module as gen_batch_mla_module
-from .attention import gen_batch_mla_tvm_binding as gen_batch_mla_tvm_binding
 from .attention import gen_batch_prefill_module as gen_batch_prefill_module
 from .attention import (
     gen_customize_batch_decode_module as gen_customize_batch_decode_module,
 )
-from .attention import (
-    gen_customize_batch_decode_tvm_binding as gen_customize_batch_decode_tvm_binding,
-)
 from .attention import (
     gen_customize_batch_prefill_module as gen_customize_batch_prefill_module,
 )
-from .attention import (
-    gen_customize_batch_prefill_tvm_binding as gen_customize_batch_prefill_tvm_binding,
-)
 from .attention import (
     gen_customize_single_decode_module as gen_customize_single_decode_module,
 )
@@ -50,7 +43,6 @@ from .attention import (
 )
 from .attention import gen_fmha_cutlass_sm100a_module as gen_fmha_cutlass_sm100a_module
 from .attention import gen_pod_module as gen_pod_module
-from .attention import gen_sampling_tvm_binding as gen_sampling_tvm_binding
 from .attention import gen_single_decode_module as gen_single_decode_module
 from .attention import gen_single_prefill_module as gen_single_prefill_module
 from .attention import get_batch_attention_uri as get_batch_attention_uri
@@ -61,22 +53,29 @@ from .attention import get_batch_prefill_uri as get_batch_prefill_uri
 from .attention import get_pod_uri as get_pod_uri
 from .attention import get_single_decode_uri as get_single_decode_uri
 from .attention import get_single_prefill_uri as get_single_prefill_uri
-from .attention import trtllm_gen_fmha_module as trtllm_gen_fmha_module
+from .attention import gen_trtllm_gen_fmha_module as gen_trtllm_gen_fmha_module
 from .core import JitSpec as JitSpec
+from .core import JitSpecStatus as JitSpecStatus
+from .core import JitSpecRegistry as JitSpecRegistry
+from .core import jit_spec_registry as jit_spec_registry
 from .core import build_jit_specs as build_jit_specs
 from .core import clear_cache_dir as clear_cache_dir
 from .core import gen_jit_spec as gen_jit_spec
+from .core import MissingJITCacheError as MissingJITCacheError
 from .core import sm90a_nvcc_flags as sm90a_nvcc_flags
 from .core import sm100a_nvcc_flags as sm100a_nvcc_flags
-from .cubin_loader import setup_cubin_loader, setup_metainfo_loader
-
-
-@functools.cache
-def get_cudnn_fmha_gen_module():
-    mod = cudnn_fmha_gen_module()
-    op = mod.build_and_load()
-    setup_cubin_loader(mod.get_library_path())
-    return op
+from .core import sm100f_nvcc_flags as sm100f_nvcc_flags
+from .core import sm103a_nvcc_flags as sm103a_nvcc_flags
+from .core import sm110a_nvcc_flags as sm110a_nvcc_flags
+from .core import sm120a_nvcc_flags as sm120a_nvcc_flags
+from .core import sm121a_nvcc_flags as sm121a_nvcc_flags
+from .core import current_compilation_context as current_compilation_context
+from .cubin_loader import setup_cubin_loader
+from .comm import gen_comm_alltoall_module as gen_comm_alltoall_module
+from .comm import gen_trtllm_mnnvl_comm_module as gen_trtllm_mnnvl_comm_module
+from .comm import gen_trtllm_comm_module as gen_trtllm_comm_module
+from .comm import gen_vllm_comm_module as gen_vllm_comm_module
+from .comm import gen_nvshmem_module as gen_nvshmem_module
 
 
 cuda_lib_path = os.environ.get(
