@@ -15,14 +15,11 @@ limitations under the License.
 """
 
 import functools
-from functools import cache
-from typing import Any, Optional, Tuple, Union
+from typing import Optional, Tuple, Union
 
 import torch
 
-from .jit import JitSpec
-from .jit import env as jit_env
-from .jit import gen_jit_spec
+from .jit.page import gen_page_module
 from .utils import (
     TensorLayout,
     _check_kv_layout,
@@ -32,16 +29,6 @@ from .utils import (
 )
 
 
-def gen_page_module() -> JitSpec:
-    return gen_jit_spec(
-        "page",
-        [
-            jit_env.FLASHINFER_CSRC_DIR / "page.cu",
-            jit_env.FLASHINFER_CSRC_DIR / "flashinfer_page_ops.cu",
-        ],
-    )
-
-
 @functools.cache
 def get_page_module():
     return gen_page_module().build_and_load()
