@@ -239,13 +239,29 @@ struct BatchedGemmData {
     // The dtype is float32.
     void const* mPtrBias{nullptr};
 
-    // The output tensor scaling factor for MxFp{4,8}, Fp8 and NvFp4 quantization.
+    // The output tensor scaling factor for Fp8 (not DeepSeek FP8) and NvFp4 quantization.
     // TensorRT-LLM API requires a scaling factor on the device.
+    // scaleC = dequantA * dequantB * quantC,
+    // where dequantA is global dequantization scaling factor of A
+    //    if dtypeA is FP8, it transforms the range from [-448, 448] to [-amaxA, amaxA]
+    //    if dtypeA is NvFp4, it transforms the range from [-448 * 6, 448 * 6] to [-amaxA, amaxA],
+    //    otherwise it is 1.
+    // dequantB is defined similarly to dequantA.
+    // quantC is the quantization scaling factor of C.
+    //    if dtypeC is FP8, it transforms the range from [-amaxC, amaxC] to [-448, 448]
+    //    if dtypeC is NvFp4, it transforms the range from [-amaxC, amaxC] to [-448 * 6, 448 * 6],
+    //    otherwise it is 1.
     // Shape is [B].
     float const* mPtrScaleC{nullptr};
 
-    // The output gate scale for MxFp{4,8} and NvFp4 quantization.
+    // The output gate scale for Fp8 (not DeepSeek FP8) and NvFp4 quantization.
     // TensorRT-LLM API requires a scaling factor on the device.
+    // scaleGate = dequantA * dequantB,
+    // where dequantA is global dequantization scaling factor of A
+    //    if dtypeA is FP8, it transforms the range from [-448, 448] to [-amaxA, amaxA]
+    //    if dtypeA is NvFp4, it transforms the range from [-448 * 6, 448 * 6] to [-amaxA, amaxA],
+    //    otherwise it is 1.
+    // dequantB is defined similarly to dequantA.
     // Shape is [B].
     float const* mPtrScaleGate{nullptr};
 
@@ -279,7 +295,7 @@ struct BatchedGemmData {
     // beta' = beta / dqAb
     // out = scaleC * (x1 + beta') * x0
     //
-    // Note this assumes that scaleAb == scaleGate which is true in TRT-LLM MoE use-case
+    // Note this assumes that dequantScaleAb == scaleGate which is true in TRT-LLM MoE use-case
     //
     float const* mPtrClampLimit{nullptr};
 
@@ -291,6 +307,10 @@ struct BatchedGemmData {
     // The formula for SwiGlu (for GeGlu, replace sigmoid with phi):
     //
     //   out_glu  = x_glu * torch.sigmoid(alpha * x_glu) * (x_linear + beta)
+    //
+    // The beta is added before applying the global scaling factor. I.e.
+    // x_linear = (x_linear + beta') * scaleC
+    // Thus, the beta' = beta / (dequantA * dequantB), where the beta is the original beta.
     float const* mPtrGatedActAlpha{nullptr};
     float const* mPtrGatedActBeta{nullptr};
 
@@ -502,13 +522,14 @@ class BatchedGemmInterface {
   // Returns true if the configuration of the cubin can be executed for the given params.
   bool isValidConfig(BatchedGemmConfig const& config, BatchedGemmData const& data) const;
 
+  // Creates GemmOptions from kernel and data.
+  BatchedGemmOptions getOptionsFromConfigAndData(BatchedGemmConfig const& config,
+                                                 BatchedGemmData const& data) const;
+
  private:
   // Aligns the pointer to the alignment
   template <typename Dtype>
   inline Dtype* alignPtr(Dtype* ptr, int64_t alignment) const;
-  // Creates GemmOptions from kernel and data.
-  BatchedGemmOptions getOptionsFromConfigAndData(BatchedGemmConfig const& config,
-                                                 BatchedGemmData const& data) const;
 
   // Returns the size of the workspace buffers in bytes
   std::vector<size_t> getWorkspaceSizesInBytes(BatchedGemmConfig const& config,
@@ -575,7 +596,7 @@ bool BatchedGemmInterface::isValidConfig(BatchedGemmConfig const& config,
   auto options = getOptionsFromConfigAndData(config, data);
 
   // Is Blackwell?
-  bool isBlackwell = config.mSm == gemm::SmVersion::Sm100a;
+  bool isBlackwell = gemm::isSmVersionBlackwell(config.mSm);
 
   // Check options without modifications.
   return checkAndUpdateBatchedGemmOptions(options, isBlackwell,
@@ -706,7 +727,7 @@ int32_t BatchedGemmInterface::run(BatchedGemmConfig const& config, void* workspa
     if (!fname_cubin.empty()) {
       fname_cubin[0] = static_cast<char>(std::toupper(static_cast<unsigned char>(fname_cubin[0])));
     }
-    fname_cubin = tllm_gen_bmm_cubin_path + fname_cubin;
+    fname_cubin = tllm_gen_bmm_cubin_path + "/" + fname_cubin + ".cubin";
     std::string cubin = flashinfer::trtllm_cubin_loader::getCubin(fname_cubin, sha256);
     cuModuleLoadData(&cuModule, cubin.c_str());
   };
